@@ -1,49 +1,5 @@
 ################################################################################
 
-#' Predict method
-#'
-#' Predict method for class `big_CMSA`.
-#'
-#' @param object Object of class `big_CMSA`.
-#' @inheritParams bigstatsr-package
-#' @param ... Not used.
-#'
-#' @return A vector of prediction scores, corresponding to `ind.row`.
-#'
-#' @export
-#' @importFrom stats predict
-#'
-#' @seealso [big_CMSA]
-#'
-predict.big_CMSA <- function(object, X,
-                             ind.row = rows_along(X),
-                             covar.row = NULL,
-                             ...) {
-
-  check_args()
-
-  ind.col <- which(object[cols_along(X)] != 0)
-  stopifnot(length(ind.col) > 0)
-
-  scores <- big_prodVec(X, object[ind.col],
-                        ind.row = ind.row,
-                        ind.col = ind.col)
-
-  if (!is.null(covar.row)) {
-
-    assert_lengths(ind.row, rows_along(covar.row))
-
-    ncov <- ncol(covar.row)
-
-    scores <- scores + as.numeric(covar.row %*% tail(object, ncov))
-  }
-
-  names(scores) <- ind.row
-  scores
-}
-
-################################################################################
-
 #' Predict method
 #'
 #' Predict method for class `big_sp`.
@@ -59,12 +15,11 @@ predict.big_CMSA <- function(object, X,
 #' @import Matrix
 #' @importFrom stats predict
 #'
-#' @seealso [big_spLinReg], [big_spLogReg] and [big_spSVM].
-#'
-#' @example examples/example-predict.R
+#' @seealso [big_spLinReg] and [big_spLogReg].
 #'
 predict.big_sp <- function(object, X,
                            ind.row = rows_along(X),
+                           ind.col = object$ind.col,
                            covar.row = NULL,
                            block.size = block_size(nrow(X)),
                            ...) {
@@ -75,20 +30,23 @@ predict.big_sp <- function(object, X,
 
   if (is.null(covar.row)) {
 
-    scores <- big_prodMat(X, betas,
+    ind.nozero <- which(rowSums(betas != 0) > 0)
+    scores <- big_prodMat(X, betas[ind.nozero, , drop = FALSE],
                           ind.row = ind.row,
+                          ind.col = ind.col[ind.nozero],
                           block.size = block.size)
 
   } else {
 
     assert_lengths(ind.row, rows_along(covar.row))
 
-    ind.X <- cols_along(X)
-    scores <- big_prodMat(X, betas[ind.X, ],
+    ind.X <- seq_along(ind.col)
+    ind.nozero <- which(rowSums(betas != 0)[ind.X] > 0)
+    scores <- big_prodMat(X, betas[ind.X[ind.nozero], ],
                           ind.row = ind.row,
+                          ind.col = ind.col[ind.nozero],
                           block.size = block.size) +
-      covar.row %*% betas[-ind.X, ]
-
+      covar.row %*% betas[-ind.X, , drop = FALSE]
   }
 
   rownames(scores) <- ind.row
@@ -97,6 +55,50 @@ predict.big_sp <- function(object, X,
 
 ################################################################################
 
+#' Predict method
+#'
+#' Predict method for class `big_sp_best_list`.
+#'
+#' @param object Object of class `big_sp_best_list`.
+#' @inheritParams bigstatsr-package
+#' @param ... Not used.
+#' @param proba Whether to return probabilities?
+#'
+#' @return A vector of scores, corresponding to `ind.row`.
+#'
+#' @export
+#' @importFrom stats predict
+#'
+#' @seealso [big_spLinReg] and [big_spLogReg].
+#'
+predict.big_sp_best_list <- function(object, X,
+                                     ind.row = rows_along(X),
+                                     ind.col = attr(object, "ind.col"),
+                                     covar.row = NULL,
+                                     proba = (attr(object, "family") == "binomial"),
+                                     ...) {
+
+  check_args()
+
+  sapply(object, function(obj) {
+    beta.X <- obj$beta.X
+    ind.nozero <- which(beta.X != 0)
+
+    scores <- big_prodVec(X, beta.X[ind.nozero],
+                          ind.row = ind.row,
+                          ind.col = ind.col[ind.nozero]) +
+      obj$intercept
+
+    if (!is.null(covar.row))
+      scores <- scores + drop(covar.row %*% obj$beta.covar)
+
+    names(scores) <- ind.row
+    `if`(proba, 1 / (1 + exp(-scores)), scores)
+  })
+}
+
+################################################################################
+
 #' Predict method
 #'
 #' Predict method for class `mhtest`.
@@ -125,7 +127,6 @@ predict.mhtest <- function(object, scores = object$score, log10 = TRUE, ...) {
   `if`(log10, lpval, 10^lpval)
 }
 
-
 ################################################################################
 
 #' Scores of PCA
